import { useEffect, useState } from 'react'
import { Formik, Form } from 'formik'
import { useUsersQuery, UsersQuery, Permission } from '../../gql'
import { TextInput, Checkbox } from '../../form'

const Permissions = Object.getOwnPropertyNames(Permission)
console.log('Permissions', Permissions)

const emptyUser = {
  name: '',
  email: '',
  interests: [],
  admin: false,
  instructor: false,
  id: ''
}

interface UserInputProps {
  inputProps: (name: string, options?: any) => any
  subtree: string
}

const UserInput = ({ path, ...props }: any) => {
  const [touched, touch] = useState(false)

  return (
    <tr>
      <td><input type='checkbox' /></td>
      <td><TextInput name={`${path}.name`} type='text' placeholder='Name' /></td>
      <td><TextInput name={`${path}.email`} type='email' placeholder='eMail' /></td>
      <td><Checkbox name={`${path}.admin`} /></td>
      <td><Checkbox name={`${path}.instructor`} /></td>
      <td></td>
      <td><button>Delete</button><button>Save</button></td>
    </tr>

  )
}

const UserAdmin = () => {
  const { data, error, loading } = useUsersQuery()

  useEffect(() => {
  }, [data])

  if (error) return <p>Error: {error.message}</p>
  if (loading) return <p>Loading...</p>
  if (data) {
    const formData = data.users.map(user => {
      if (!user) return null
      const { permissions, ...rest } = user
      return {
        ...rest,
        admin: permissions.includes(Permission.Admin),
        instructor: permissions.includes(Permission.Instructor)
      }
    })

    return (
      <Formik
        initialValues={formData}
        onSubmit={values => console.log(values)}
      >
        {({ values, setValues }) => (
          <>
            <Form>
              <table>
                <thead>
                  <tr>
                    <th></th>
                    <th>Name</th>
                    <th>Email</th>
                    {Permissions.map(permission => (
                      <th key={permission}>{permission}</th>
                    ))}
                    <th>Interests</th>
                    <th>Actions</th>
                  </tr>
                </thead>
                <tbody>
                  {values.map((user, index) => user && (
                    <UserInput path={`[${index}]`} key={user.id} />
                  ))}
                </tbody>
              </table>
            </Form>
            <button onClick={() => {
              setValues([
                ...values,
                { ...emptyUser }
              ])
            }}>Add user</button>
          </>
        )}
      </Formik>
    )
  }
}

export default UserAdmin
